##############################################################################
#
# Copyright (c) 2010 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
from HTMLParser import HTMLParser
from paste.script import templates
from paste.script.templates import var
from urllib2 import urlopen
import pkg_resources
import re, os, sys

DOWNLOAD_URL = 'http://download.ztfy.org/webapp/'

class FindLatest(HTMLParser):
    """html parser used to find the latest version file on download.zope.org
    """
    def reset(self):
        """initialize a regexp and a version set"""
        HTMLParser.reset(self)
        self.regexp = re.compile('.*ztfy.webapp-(.*).cfg$')
        self.versions = set()

    def handle_starttag(self, tag, attrs):
        """extract the version from each link"""
        if tag != 'a':
            return
        for attr in attrs:
            if attr[0] == 'href' and self.regexp.match(attr[1]):
                self.versions.add(self.regexp.sub('\\1', attr[1]))


class Webapp(templates.Template):
    """ The main paster template for ZTFY.webapp
    """
    _template_dir = 'webapp_template'
    summary = "A ZTFY webapp project, referencing all ZTFY packages"

    vars = [
        var('python_package',
            u'Main Python package (without namespace)'),
        var('interpreter',
            u'Name of custom Python interpreter',
            default='ztpy'),
        var('release', (u'Which release of ZTFY.webapp?\n'
                        u'Check on %s' % DOWNLOAD_URL)),
        var('version', u'Version of your project', default='0.1'),
        var('description', u'One-line description of the package'),
        var('long_description', u'Multi-line description (in reST)'),
        var('keywords', u'Space-separated keywords/tags'),
        var('author', u'Author name'),
        var('author_email', u'Author email'),
        var('url', u'URL of homepage'),
        var('license_name', u'License name'),
        ]

    def check_vars(self, vars, cmd):
        """This method checks the variables and ask for missing ones
        """
        # Find the latest versions.cfg online
        current = pkg_resources.get_distribution('ztfy.webapp').version
        latest = current
        try:
            if 'offline' not in vars:  #offline is used in tests
                sys.stdout.write('Searching the latest version... ')
                sys.stdout.flush()
                parse_version = pkg_resources.parse_version

                # download and parse the html page and store versions
                parser = FindLatest()
                parser.feed(urlopen(DOWNLOAD_URL).read())

                # return the highest minor version
                if not len(parser.versions):
                    raise IOError('No versions found')
                all_versions = sorted(parser.versions,
                                      key=lambda v: parse_version(v),
                                      reverse=True)
                for v in all_versions:
                    trunked_v = [x for x in parse_version(v)[:2] if not x.startswith('*')]
                    trunked_current = [x for x in parse_version(current)[:2] if not x.startswith('*')]
                    while len(trunked_v) < 2:
                        trunked_v.append('00000000')
                    while len(trunked_current) < 2:
                        trunked_current.append('00000000')
                    if trunked_v > trunked_current:
                        continue
                    latest = v
                    break

                print str(latest) + '\n'

                # warn the user if there is a change in the latest template
                last_change = '1.1.0' # feature introduced for 1.0b4
                for line in urlopen(DOWNLOAD_URL
                                    + 'ztfy.webapp-%s.cfg' % latest).readlines():
                    if 'LAST_TEMPLATE_CHANGE' in line:
                        last_change = line.split('=')[1].strip()
                        break
                if parse_version(last_change) > parse_version(current):
                    print ('**WARNING**: the project template for ZTFY.webapp '
                           'has changed since version %s.\n'
                           'You should upgrade to this version.\n' % last_change)

        except IOError:
            # if something wrong occurs, we keep the current version
            print u'**WARNING**: error while getting the latest version online'
            print u'Please check that you can access %s\n' % DOWNLOAD_URL

        # suggest what Paste chose
        for var in self.vars:
            if var.name == 'release':
                var.default = latest
            if var.name == 'python_package':
                var.default = vars['package']
                # but keep the user's input if there are dots in the middle
                if '.' in vars['project'][1:-1]:
                    var.default = re.sub('[^A-Za-z0-9.]+', '_',
                                         vars['project']).lower()

        # ask remaining variables
        vars = templates.Template.check_vars(self, vars, cmd)

        # replace Paste default choice with ours
        vars['package'] = vars['python_package']

        # check for bad python package names
        if vars['package'] in ('bluebream', 'bream', 'zope'):
            print
            print "Error: The chosen project name results in an invalid " \
                  "package name: %s." % vars['package']
            print "Please choose a different project name."
            sys.exit(1)
        vars['zip_safe'] = False
        # remember the version of bluebream used to create the project
        vars['created_with'] = current
        return vars

    def pre(self, command, output_dir, vars):
        """Detect namespaces in the project name
        """
        if not command.options.verbose:
            command.verbose = 0
        self.ns_split = vars['package'].split('.')
        vars['package'] = self.ns_split[-1]
        vars['namespace_packages'] = list(reversed([
                    vars['python_package'].rsplit('.', i)[0]
                    for i in range(1, len(self.ns_split))]))
        vars['ns_prefix'] = '.'.join(self.ns_split[:-1]) + '.'
        if len(self.ns_split) == 1:
            vars['ns_prefix'] = ''
        vars['basedir'] = os.getcwd()

    def post(self, command, output_dir, vars):
        """Add nested namespace levels
           and move the main package to the last level
        """
        # if we have a namespace package
        if len(self.ns_split) > 1:
            package_dir = os.path.join(output_dir, 'src', vars['package'])
            tmp_dir = package_dir + '.tmp'
            os.rename(package_dir, tmp_dir)

            # create the intermediate namespace packages
            target_dir = os.path.join(output_dir, 'src',
                                      os.path.join(*self.ns_split[:-1]))
            os.makedirs(target_dir)

            # create each __init__.py with namespace declaration
            ns_decl = "__import__('pkg_resources').declare_namespace(__name__)"
            for i, namespace_package in enumerate(self.ns_split[:-1]):
                init_file = os.path.join(output_dir, 'src',
                                         os.path.join(*self.ns_split[:i + 1]),
                                         '__init__.py')
                open(init_file, 'w').write(ns_decl)

            # move the (renamed) main package to the last namespace
            os.rename(tmp_dir,
                      os.path.join(target_dir, os.path.basename(package_dir,)))

        print("\nYour project has been created! Now, you want to:\n"
              "1) put the generated files under version control\n"
              "2) run: python bootstrap.py\n"
              "3) run: ./bin/buildout\n"
             )



### -*- coding: utf-8 -*- ####################################################
##############################################################################
#
# Copyright (c) 2008-2012 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################

__docformat__ = "restructuredtext"

# import standard packages

# import Zope3 interfaces

# import local interfaces

# import Zope3 packages
import zope.app.wsgi

# import local packages
try:
    import psyco
    psyco.profile()
except:
    pass


def application_factory(global_conf):
    zope_conf = global_conf['zope_conf']
    app = zope.app.wsgi.getWSGIApplication(zope_conf)

    def wrapper(environ, start_response):
        vhost = ''
        vhost_skin = environ.get('HTTP_X_VHM_SKIN')
        if vhost_skin and \
           (not environ.get('CONTENT_TYPE', '').startswith('application/json')) and \
           (not environ.get('PATH_INFO', '').startswith('/++skin++')):
            vhost = '/++skin++' + vhost_skin
        url_scheme = environ.get('wsgi.url_scheme', 'http')
        vhost_root = environ.get('HTTP_X_VHM_ROOT', '')
        if (url_scheme == 'https') or (vhost_root and (vhost_root != '/')):
            # Standard virtual host URL rewriting
            # Also required for HTTPS connexions
            vhost += '%s/++vh++%s:%s:%s/++' % (vhost_root,
                                               url_scheme,
                                               environ.get('SERVER_NAME', ''),
                                               environ.get('SERVER_PORT', '80'))
        environ['PATH_INFO'] = vhost + environ['PATH_INFO']
        return app(environ, start_response)

    return wrapper



import os
from paste.script import command
from paste.deploy import appconfig
from zope.app.debug.debug import Debugger
import zope.app.wsgi


class Shell(command.Command):

    max_args = 1
    min_args = 1

    usage = "CONFIG_FILE"
    summary = "Python debug shell with BlueBream application loaded"
    group_name = "bluebream"

    parser = command.Command.standard_parser(verbose=True)

    def command(self):
        cwd = os.getcwd()
        config_file = self.args[0]
        config_name = 'config:%s' % config_file
        conf = appconfig(config_name, relative_to=cwd)
        zope_conf = conf['zope_conf']
        db = zope.app.wsgi.config(zope_conf)
        debugger = Debugger.fromDatabase(db)
        # Invoke an interactive interpreter shell
        banner = ("Welcome to the interactive debug prompt.\n"
                  "The 'root' variable contains the ZODB root folder.\n"
                  "The 'app' variable contains the Debugger, 'app.publish(path)' "
                  "simulates a request.")
        __import__('code').interact(banner=banner,
                                    local={'debugger': debugger,
                                           'app': debugger,
                                           'root': debugger.root()})



import os
from paste.script import pluginlib
from paste.script.pluginlib import egg_name
from paste.script import create_distro
from paste.script import copydir

def egg_info_dir(base_dir, dist_name):
    all = []
    for dir_extension in ['.'] + os.listdir(base_dir):
        full = os.path.join(base_dir, dir_extension,
                            egg_name(dist_name)+'.egg-info')
        all.append(full)
        if os.path.exists(full):
            return full
    return ''

pluginlib.egg_info_dir = egg_info_dir

def command(self):
    if self.options.list_templates:
        return self.list_templates()
    asked_tmpls = self.options.templates or ['basic_package']
    templates = []
    for tmpl_name in asked_tmpls:
        self.extend_templates(templates, tmpl_name)
    if self.options.list_variables:
        return self.list_variables(templates)
    if self.verbose:
        print 'Selected and implied templates:'
        max_tmpl_name = max([len(tmpl_name) for tmpl_name, tmpl in templates])
        for tmpl_name, tmpl in templates:
            print '  %s%s  %s' % (
                tmpl_name, ' '*(max_tmpl_name-len(tmpl_name)),
                tmpl.summary)
        print
    if not self.args:
        if self.interactive:
            dist_name = self.challenge('Enter project name')
        else:
            raise BadCommand('You must provide a PACKAGE_NAME')
    else:
        dist_name = self.args[0].lstrip(os.path.sep)

    templates = [tmpl for name, tmpl in templates]
    output_dir = os.path.join(self.options.output_dir, dist_name)
    
    pkg_name = self._bad_chars_re.sub('', dist_name.lower())
    vars = {'project': dist_name,
            'package': pkg_name,
            'egg': pluginlib.egg_name(dist_name),
            }
    vars.update(self.parse_vars(self.args[1:]))
    if self.options.config and os.path.exists(self.options.config):
        for key, value in self.read_vars(self.options.config).items():
            vars.setdefault(key, value)
    
    if self.verbose: # @@: > 1?
        self.display_vars(vars)

    if self.options.inspect_files:
        self.inspect_files(
            output_dir, templates, vars)
        return
    if not os.path.exists(output_dir):
        # We want to avoid asking questions in copydir if the path
        # doesn't exist yet
        copydir.all_answer = 'y'
    
    if self.options.svn_repository:
        self.setup_svn_repository(output_dir, dist_name)

    # First we want to make sure all the templates get a chance to
    # set their variables, all at once, with the most specialized
    # template going first (the last template is the most
    # specialized)...
    for template in templates[::-1]:
        vars = template.check_vars(vars, self)

    # Gather all the templates egg_plugins into one var
    egg_plugins = set()
    for template in templates:
        egg_plugins.update(template.egg_plugins)
    egg_plugins = list(egg_plugins)
    egg_plugins.sort()
    vars['egg_plugins'] = egg_plugins
        
    for template in templates:
        self.create_template(
            template, output_dir, vars)

    found_setup_py = False
    paster_plugins_mtime = None

    package_dir = vars.get('package_dir', None)
    if package_dir:
        output_dir = os.path.join(output_dir, package_dir)

    # With no setup.py this doesn't make sense:
    if found_setup_py:
        # Only write paster_plugins.txt if it wasn't written by
        # egg_info (the correct way). leaving us to do it is
        # deprecated and you'll get warned
        egg_info_dir = pluginlib.egg_info_dir(output_dir, dist_name)
        plugins_path = os.path.join(egg_info_dir, 'paster_plugins.txt')
        if len(egg_plugins) and (not os.path.exists(plugins_path) or \
                os.path.getmtime(plugins_path) == paster_plugins_mtime):
            if self.verbose:
                print >> sys.stderr, \
                    ('Manually creating paster_plugins.txt (deprecated! '
                     'pass a paster_plugins keyword to setup() instead)')
            for plugin in egg_plugins:
                if self.verbose:
                    print 'Adding %s to paster_plugins.txt' % plugin
                if not self.simulate:
                    pluginlib.add_plugin(egg_info_dir, plugin)
    
    if self.options.svn_repository:
        self.add_svn_repository(vars, output_dir)

    if self.options.config:
        write_vars = vars.copy()
        del write_vars['project']
        del write_vars['package']
        self.write_vars(self.options.config, write_vars)
 
create_distro.CreateDistroCommand.command = command



##############################################################################
#
# Copyright (c) 2010 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################

import re
import doctest
import zc.buildout.testing
from zope.testing import renormalizing


def easy_install_SetUp(test):
    zc.buildout.testing.buildoutSetUp(test)


def test_suite():
    return doctest.DocFileSuite(
        'webapp.txt',
        setUp=easy_install_SetUp,
        tearDown=zc.buildout.testing.buildoutTearDown,
        optionflags=doctest.ELLIPSIS | doctest.NORMALIZE_WHITESPACE,
        checker=renormalizing.RENormalizing([
                zc.buildout.testing.normalize_endings,
                zc.buildout.testing.normalize_script,
                zc.buildout.testing.normalize_egg_py,
                (re.compile('Running .*python.* setup.py'), 'Running python setup.py'),
                (re.compile(r'\\'), '/') #windows path happiness
                ]))



##############################################################################
#
# Copyright (c) 2010 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################

import os
from setuptools import setup, find_packages

def read(*rnames):
    return open(os.path.join(os.path.dirname(__file__), *rnames)).read()

setup(
    name="ztfy.webapp",
    version='1.1.2',
    author="Thierry Florac",
    author_email="tflorac@ulthar.net",
    url="http://www.ztfy.org",
    description="ZTFY web application template",
    long_description=(
        read('README.txt')
        + '\n\n' +
        read('CHANGES.txt')
        + '\n\n' +
        'Download\n'
        '********'),
    license="ZPL",
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Zope Public License',
        'Programming Language :: Python',
        'Framework :: Zope3',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: WSGI',
        ],
    packages=find_packages("src"),
    namespace_packages=['ztfy'],
    package_dir={"": "src"},
    package_data={"": [ '*.txt', '*.cfg', '*_tmpl', ]},
    zip_safe=False,
    include_package_data=True,
    install_requires=["setuptools", "PasteScript>=1.7.3"],
    extras_require={"test": ["zc.buildout", "zope.testing"]},
    entry_points={
        "paste.paster_create_template": [ "ztfy.webapp = ztfy.webapp.webapp_base.template:Webapp", ]
    })

